
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _DESERIALIZER_HH_
#define _DESERIALIZER_HH_

/**
 * @file deserializer.hh
 * Deserialize structures and types from input on a stream.
 */

#include <istream>
#include <string>
#include <stdexcept>

#include <boost/shared_ptr.hpp>


namespace dc {


class serializable;
typedef boost::shared_ptr<serializable> serializable_ptr;

class deserializer_impl;

class deserializer
{
public:
	deserializer(const std::string& filePath, bool comments = false,
			bool check = false);
	deserializer(std::istream& input, bool comments = false, bool check = false);

	serializable_ptr deserialize();

	serializable* pullNext();
	void pop();

	struct exception : std::runtime_error
	{
		explicit exception(const std::string& what_arg) :
			std::runtime_error(what_arg) {}
	};

private:
	boost::shared_ptr<deserializer_impl> impl;
};


} // namespace dc


#endif // _DESERIALIZER_HH_

