
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _RESOURCE_HH_
#define _RESOURCE_HH_

/**
 * @file resource.hh
 * Parent class of textures, sounds, other assets.
 */

#include <stdexcept>
#include <string>
#include <vector>


namespace dc {


/**
 * Generic resource class.  
 */

class resource
{
public:
	struct exception : public std::runtime_error
	{
		explicit exception(const std::string& what_arg) :
			std::runtime_error(what_arg) {}
	};

	resource(const std::string& name) throw(exception);
	virtual ~resource();

	/**
	 * Get the path of the file associated with this resource.
	 * @return Path.
	 */

	const std::string& getPathToFile();


	/**
	 * Add a directory to search when looking for resource files.
	 * @param directory Path to a directory.
	 */

	static void addSearchPath(const std::string& directory);

	/**
	 * Get the path to a resource of a given name.
	 * @param name Name of a resource.  The name will be appended to each search
	 * directory in order.
	 * @return The first path found which resolves to a file.
	 */

	static std::string getPathToResource(const std::string& name);

private:
	std::string filePath_;
	static std::vector<std::string> searchPaths_;
};


} // namespace dc

#endif // _RESOURCE_HH_

