
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _SETTINGS_HH_
#define _SETTINGS_HH_

/**
 * @file settings.hh
 * Load, store, save program settings.
 */

#include <string>
#include <map>

#include "singleton.hh"
#include "serializable.hh"


namespace dc {


class settings : public singleton<settings>
{
public:
	settings() {}
	settings(int argc, char* argv[]);

	void parseArgs(int argc, char* argv[]);
	void loadFromFile(std::string filePath);

	template <typename T>
	bool get(const std::string& key, T& value);

private:
	std::map<std::string,serializable_ptr> map;
};


template <typename T>
bool settings::get(const std::string& key, T& value)
{
	std::map<std::string,serializable_ptr>::const_iterator i = map.find(key);

	if (i != map.end())
	{
		serializable_ptr obj = (*i).second;
		return obj->get(value);
	}
	else
	{
		return false;
	}
}


} // namepsace dc

#endif // _SETTINGS_HH_

