
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _VIDEO_HH_
#define _VIDEO_HH_

#include <string>

#include <boost/shared_ptr.hpp>

#include <SDL/SDL.h>


namespace dc {


class video
{
public:
	struct attributes
	{
		// OpenGL attributes
		long	colorBuffer[4];	// rgba
		long	frameBuffer;
		bool	doubleBuffer;
		long	depthBuffer;
		long	stencilBuffer;
		long	accumBuffer[4];	// rgba
		bool	stereo;
		long	multisampleBuffers;
		long	multisampleSamples;
		bool	swapControl;
		bool	hardwareonly;

		// Window attributes
		long	mode[3];		// width, height, bpp
		bool	fullscreen;
		bool	resizable;
		bool	cursorVisible;
		bool	cursorGrab;

		attributes();
	};

	video();
	video(const attributes& attribs);
	video(const attributes& attribs, const std::string& caption);
	video(const std::string& caption);
	~video();

	void setVideoMode(const long mode[3]);
	attributes getAttributes() const;

	void resize(int width, int height);
	bool iconify();

	void setCaption(const std::string& caption);
	std::string getCaption() const;

	void setFull(bool full);
	void toggleFull();
	bool isFull() const;

	void setCursorVisible(bool hasCursor);
	void toggleCursorVisible();
	bool isCursorVisible() const;

	void setResizable(bool resizable);
	void toggleResizable();
	bool isResizable() const;

	void setCursorGrab(bool cursorGrab);
	void toggleCursorGrab();
	bool isCursorGrab() const;

	void makeActive();
	void swap();

private:
	void init(const attributes& attribs, const std::string& caption);
	void recreateContext();
	void setOpenGLAttributes();

	SDL_Surface* context_;
	unsigned flags_;
	attributes attribs_;
};

typedef boost::shared_ptr<video> video_ptr;


} // namespace dc

#endif // _VIDEO_HH_

